@@ -3,14 +3,14 @@
 from sql import Null, Literal
 from sql.functions import CurrentTimestamp
 
-import stdnum.eu.at_02 as sepa
-import stdnum.exceptions
 
+from trytond.i18n import gettext
 from trytond.pool import PoolMeta, Pool
 from trytond.model import fields
 from trytond.transaction import Transaction
+from trytond.pyson import Eval
 
-__all__ = ['Party', 'PartyIdentifier']
+from .exceptions import PartyIdentificationdError
 
 
 class Party(metaclass=PoolMeta):
@@ -43,53 +43,68 @@ class Party(metaclass=PoolMeta):
                     values=select))
             table.drop_column('sepa_creditor_identifier')
 
+    @classmethod
+    def copy(cls, parties, default=None):
+        if default is None:
+            default = {}
+        else:
+            default = default.copy()
+        default.setdefault('sepa_mandates', [])
+        return super().copy(parties, default=default)
+
     def get_sepa_creditor_identifier_used(self, name):
         for identifier in self.identifiers:
-            if identifier.type == 'sepa':
+            if identifier.type == 'eu_at_02':
                 return identifier.code
 
+    def get_sepa_identifier(self, name):
+        pool = Pool()
+        Identifier = pool.get('party.identifier')
+        for identifier in self.identifiers:
+            if identifier.type == name:
+                return identifier.sepa_identifier
+        else:
+            selection = Identifier.fields_get(['type'])['type']['selection']
+            type = dict(selection).get(name, name)
+            raise PartyIdentificationdError(
+                gettext('account_payment_sepa.msg_party_no_id',
+                    party=self.rec_name,
+                    type=type))
+
 
 class PartyIdentifier(metaclass=PoolMeta):
     __name__ = 'party.identifier'
 
+    sepa_es_suffix = fields.Char(
+        "SEPA Suffix", size=3,
+        states={
+            'invisible': Eval('type') != 'es_nif',
+            },
+        depends=['type'])
+
     @classmethod
-    def __setup__(cls):
-        super(PartyIdentifier, cls).__setup__()
-        cls.type.selection.append(('sepa', 'SEPA Creditor Identifier'))
-        cls._error_messages.update({
-                'unique_sepa': ('Party "%(party)s" has more than one '
-                    'SEPA Creditor Identifier.'),
-                'sepa_invalid': ('The SEPA identifier "%(code)s" on party '
-                    '"%(party)s" is not valid.'),
-                })
+    def __register__(cls, module_name):
+        cursor = Transaction().connection.cursor()
+        sql_table = cls.__table__()
+        super().__register__(module_name)
 
-    @fields.depends('party', '_parent_party.identifiers')
-    def check_code(self):
-        super(PartyIdentifier, self).check_code()
-        if self.type == 'sepa':
-            for identifier in self.party.identifiers:
-                if identifier.type == 'sepa' and self != identifier:
-                    self.raise_user_error('unique_sepa', {
-                            'invalid_sepa': self.code,
-                            'party': self.party.rec_name,
-                            })
-            if not sepa.is_valid(self.code):
-                # Called from pre_validate so party may not be saved yet
-                if self.party and self.party.id > 0:
-                    party = self.party.rec_name
-                else:
-                    party = ''
-                self.raise_user_error('sepa_invalid', {
-                        'code': self.code,
-                        'party': party,
-                        })
+        # Migration from 5.4: sepa identifier merged into eu_at_02
+        cursor.execute(*sql_table.update(
+                columns=[sql_table.type],
+                values=['eu_at_02'],
+                where=sql_table.type == 'sepa'))
 
-    @fields.depends('type', 'code')
-    def on_change_with_code(self):
-        code = super(PartyIdentifier, self).on_change_with_code()
-        if self.type == 'sepa':
-            try:
-                return sepa.compact(self.code)
-            except stdnum.exceptions.ValidationError:
-                pass
-        return code
+    @property
+    def sepa_identifier(self):
+        identifier = {
+            'Type': 'OrgId',
+            'Id': self.code,
+            }
+        if self.type == 'eu_at_02':
+            identifier['Type'] = 'PrvtId'
+            identifier['SchmeNm'] = {'Prtry': 'SEPA'}
+        elif self.type == 'be_vat':
+            identifier['Issr'] = 'KBO-BCE'
+        elif self.type == 'es_nif':
+            identifier['Id'] += self.sepa_es_suffix or '000'
+        return identifier
